from typing import Dict, List, Any
from requests import HTTPError, Response
from urllib.parse import urlparse, parse_qs
from functools import reduce


class CoinMetricsUnauthorizedException(HTTPError):
    """
    Raised when a request is made that will return an error due to being unauthorized to flat files server
    """

    def __init__(self, response: Response, *args: Any, **kwargs: Any):
        if response.status_code not in [401, 403]:
            response.raise_for_status()
        self.response = response
        self.request = response.request
        error_message = """The provided API key is not authorized to access the Coin Metrics Flat Files server. This product is separate from the API. If you'd like access granted or believe this is a mistake please contact Coin Metrics support.
                    """
        self.msg = error_message
        super().__init__(response=response, request=response.request, *args, **kwargs)

    def __str__(self) -> str:
        return self.msg


class CoinMetricsClientQueryParamsException(HTTPError):
    """
    Raised when a request is made that will return an error due to the logic or contents of the request
    """

    def __init__(self, response: Response, *args: Any, **kwargs: Any):
        if response.status_code != 414:
            response.raise_for_status()
        parsed_query_params: Dict[str, List[str]] = parse_qs(
            str(urlparse(url=response.request.url).query)
        )
        get_sum_of_lengths = lambda strings: reduce(lambda a, b: a + len(b), strings, 0)
        param_length_dict = {"Total characters": 0}
        for param, values in parsed_query_params.items():
            sum_of_param_lengths = get_sum_of_lengths(values)
            param_length_dict[param] = sum_of_param_lengths
            param_length_dict["Total characters"] += sum_of_param_lengths
        exception_message = (
            "This request failed because the request URL is too long, consider reducing the length "
            f"of the params.\n 414 errors may get returned as total characters in query params exceed 5000"
            f"\nLength of the params provided for reference:\n {param_length_dict}"
        )
        self.msg = exception_message
        super().__init__(*args, **kwargs)

    def __str__(self) -> str:
        return self.msg


class CoinMetricsClientNotFoundError(Exception):
    """Raised when a CoinMetricsClient instance is not found."""
    def __init__(self, message="CoinMetricsClient not found"):
        self.message = message
        super().__init__(self.message)



# %%
import time
import os
import pandas as pd
import pytest
from coinmetrics.api_client import CoinMetricsClient
from coinmetrics._utils import get_keys_from_catalog
from typing import Tuple, Any, Callable, Dict, no_type_check

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")), verbose=True)
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@no_type_check
def benchmark_endpoint(
        endpoint_method: Any, output_method: str, **endpoint_method_kwargs: Dict[Any, Any]
) -> Tuple[Any, float, int]:
    start_time = time.time()

    if output_method == "to_dataframe":
        response = endpoint_method(**endpoint_method_kwargs).to_dataframe()
    elif output_method == "to_list":
        response = endpoint_method(**endpoint_method_kwargs).to_list()

    end_time = time.time()
    response_time = end_time - start_time
    row_count = len(response)
    return response, response_time, row_count


test_cases = [
    # ("/catalog-v2/asset-metrics", client.catalog_asset_metrics_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/exchange-metrics", client.catalog_exchange_metrics_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/exchange-asset-metrics", client.catalog_exchange_asset_metrics_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/pair-metrics", client.catalog_pair_metrics_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/institution-metrics", client.catalog_institution_metrics_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-trades", client.catalog_market_trades_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-candles", client.catalog_market_candles_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-orderbooks", client.catalog_market_orderbooks_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-quotes", client.catalog_market_quotes_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-funding-rates", client.catalog_market_funding_rates_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-funding-rates-predicted", client.catalog_full_market_funding_rates_predicted_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-contract-prices", client.catalog_market_contract_prices_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-implied-volatility", client.catalog_market_implied_volatility_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-openinterest", client.catalog_market_open_interest_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-liquidtations", client.catalog_market_liquidations_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/market-metrics", client.catalog_market_metrics_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/pair-candles", client.catalog_pair_candles_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/index-levels", client.catalog_index_levels_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/index-candles", client.catalog_index_candles_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/asset-chains", client.catalog_asset_chains_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/mempool-feerates", client.catalog_mempool_feerates_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/mining-pool-tips-summary", client.catalog_mining_pool_tips_summaries_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    # ("/catalog-v2/transaction-tracker", client.catalog_transaction_tracker_assets_v2, "to_dataframe", "json", {"page_size": 10000, format: "json"}),
    ("/catalog-v2/asset-metrics", client.catalog_asset_metrics_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/exchange-metrics", client.catalog_exchange_metrics_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/exchange-asset-metrics", client.catalog_exchange_asset_metrics_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/pair-metrics", client.catalog_pair_metrics_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/institution-metrics", client.catalog_institution_metrics_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-trades", client.catalog_market_trades_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-candles", client.catalog_market_candles_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-orderbooks", client.catalog_market_orderbooks_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-quotes", client.catalog_market_quotes_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-funding-rates", client.catalog_market_funding_rates_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-funding-rates-predicted", client.catalog_full_market_funding_rates_predicted_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-contract-prices", client.catalog_market_contract_prices_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-implied-volatility", client.catalog_market_implied_volatility_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-openinterest", client.catalog_market_open_interest_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-liquidtations", client.catalog_market_liquidations_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/market-metrics", client.catalog_market_metrics_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/pair-candles", client.catalog_pair_candles_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/index-levels", client.catalog_index_levels_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/index-candles", client.catalog_index_candles_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/asset-chains", client.catalog_asset_chains_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/mempool-feerates", client.catalog_mempool_feerates_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/reference-data/assets", client.reference_data_assets, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/exchanges", client.reference_data_exchanges, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/markets", client.reference_data_markets, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/indexes", client.reference_data_indexes, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/pairs", client.reference_data_pairs, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/asset-metrics", client.reference_data_asset_metrics, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/exchange-metrics", client.reference_data_exchange_metrics, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/exchange-asset-metrics", client.reference_data_exchange_asset_metrics, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/pair-metrics", client.reference_data_pair_metrics, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/institution-metrics", client.reference_data_institution_metrics, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    ("/reference-data/market-metrics", client.reference_data_market_metrics, "to_list", "json",
     {"page_size": 10000, "format": "json"}),
    # ("/catalog-v2/mining-pool-tips-summary", client.catalog_mining_pool_tips_summaries_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    # ("/catalog-v2/transaction-tracker", client.catalog_transaction_tracker_assets_v2, "to_list", "json", {"page_size": 10000, "format": "json"}),
    ("/catalog-v2/asset-metrics", client.catalog_asset_metrics_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/exchange-metrics", client.catalog_exchange_metrics_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/exchange-asset-metrics", client.catalog_exchange_asset_metrics_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/pair-metrics", client.catalog_pair_metrics_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/institution-metrics", client.catalog_institution_metrics_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-trades", client.catalog_market_trades_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-candles", client.catalog_market_candles_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-orderbooks", client.catalog_market_orderbooks_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-quotes", client.catalog_market_quotes_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-funding-rates", client.catalog_market_funding_rates_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-funding-rates-predicted", client.catalog_full_market_funding_rates_predicted_v2, "to_list",
     "json_stream", {}),
    ("/catalog-v2/market-contract-prices", client.catalog_market_contract_prices_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-implied-volatility", client.catalog_market_implied_volatility_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-openinterest", client.catalog_market_open_interest_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-liquidtations", client.catalog_market_liquidations_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/market-metrics", client.catalog_market_metrics_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/pair-candles", client.catalog_pair_candles_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/index-levels", client.catalog_index_levels_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/index-candles", client.catalog_index_candles_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/asset-chains", client.catalog_asset_chains_v2, "to_list", "json_stream",
     {}),
    ("/catalog-v2/mempool-feerates", client.catalog_mempool_feerates_v2, "to_list", "json_stream",
     {}),
    ("/reference-data/assets", client.reference_data_assets, "to_list", "json_stream",
     {}),
    ("/reference-data/exchanges", client.reference_data_exchanges, "to_list", "json_stream",
     {}),
    ("/reference-data/markets", client.reference_data_markets, "to_list", "json_stream",
     {}),
    ("/reference-data/indexes", client.reference_data_indexes, "to_list", "json_stream",
     {}),
    ("/reference-data/pairs", client.reference_data_pairs, "to_list", "json_stream",
     {}),
    ("/reference-data/asset-metrics", client.reference_data_asset_metrics, "to_list", "json_stream",
     {}),
    ("/reference-data/exchange-metrics", client.reference_data_exchange_metrics, "to_list", "json_stream",
     {}),
    ("/reference-data/exchange-asset-metrics", client.reference_data_exchange_asset_metrics, "to_list", "json_stream",
     {}),
    ("/reference-data/pair-metrics", client.reference_data_pair_metrics, "to_list", "json_stream",
     {}),
    ("/reference-data/institution-metrics", client.reference_data_institution_metrics, "to_list", "json_stream",
     {}),
    ("/reference-data/market-metrics", client.reference_data_market_metrics, "to_list", "json_stream",
     {}),
    # ("/catalog-v2/mining-pool-tips-summary", client.catalog_mining_pool_tips_summaries_v2, "to_list", "json_stream",
    #  {}),
    # ("/catalog-v2/transaction-tracker", client.catalog_transaction_tracker_assets_v2, "to_list", "json_stream",
    #  {})
]


@no_type_check
@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def benchmark_results() -> None:
    benchmark_results = []

    for name, endpoint_method, output_method, fmt, endpoint_method_kwargs in test_cases:
        print(f"Requesting {name} output method {output_method} and format {fmt}.")
        response, response_time, row_count = benchmark_endpoint(
            endpoint_method, output_method, **endpoint_method_kwargs
        )
        benchmark_results.append((name, output_method, fmt, response_time, row_count))

    # %%
    print(f"{'Endpoint':<60} {'Output Method':<20} {'Response Time':>23} {'Rows':>10}")
    print("-" * 95)
    for name, output_method, fmt, response_time, row_count in benchmark_results:
        print(f"{name:<60} {fmt:<20} {response_time:>15.2f} seconds {row_count:>10}")

    # %%
    results_df = pd.DataFrame(benchmark_results, columns=["Endpoint", "Output Method", "Format", "Response Time", "Rows"])
    print(results_df)
    # results_df.to_csv("benchmark_results.csv", index=False)



import pytest

from coinmetrics.api_client import CoinMetricsClient
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")))
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_timeseries_pair_candles() -> None:
    """
    Tests the timeseries/pair-candles endpoints works as expected - data is returned and the date is in the right format
    """
    data = client.get_pair_candles(pairs=["btc-usd", "eth-usd", "sol-usd"])
    assert len(data.first_page()) != 0
    expected_cols = [
        "pair",
        "time",
        "price_open",
        "price_close",
        "price_high",
        "price_low",
    ]
    assert all(col in expected_cols for col in data.first_page()[0])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_transactions_v2_eth() -> None:
    """
    Tests the timeseries get list of transactions v2 for eth
    """
    transactions = client.get_list_of_transactions_v2(
        asset="eth", start_height=16644700, end_height=16644767
    ).to_list()
    assert len(transactions) >= 9800
    transactions_called_by_id = client.get_list_of_transactions_v2(
        asset="eth",
        txids="1ec9982bee6cd96049b0ac7745df4374bcd37dce996bae46d09c3d25c5cfd413",
    ).first_page()[0]
    assert (
        transactions_called_by_id["height"] == "16644767"
        and transactions_called_by_id["n_balance_updates"] == "9"
    )


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_list_of_balance_updates_v2() -> None:
    asset = "usdc"
    accounts = [
        "4a30ff596cc84c630b2998012b0586ff36113cba",
        "4dd384f41f3e91f2e204fd462dc0fa73aef029d4",
    ]
    updates = client.get_list_of_balance_updates_v2(
        asset=asset, accounts=accounts, limit_per_account=1
    ).to_list()
    assert len(updates) == 2


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_asset_profiles() -> None:
    data = client.get_asset_profiles().first_page()
    print(data)
    assert len(data) > 10
    one_inch = client.get_asset_profiles(assets="1inch").first_page()[0]
    assert one_inch["asset"] == "1inch"

@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_predicted_market_funding_rates() -> None:
    data = client.get_predicted_market_funding_rates(
        markets="bybit-1000000VINUUSDT-future"
    ).first_page()
    print(data)
    assert len(data) > 1
    predicted_funding_rate = data[0]
    assert predicted_funding_rate["market"] == "bybit-1000000VINUUSDT-future"


if __name__ == "__main__":
    pytest.main()



import os
import threading
import pytest

from coinmetrics.api_client import CoinMetricsClient

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")))
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)

COMMON_REFD_COLS = ["metric",
                    "full_name",
                    "description",
                    "product",
                    "category",
                    "subcategory",
                    "unit",
                    "data_type",
                    "type",
                    "display_name"]

@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_asset_pairs() -> None:
    expected_cols = COMMON_REFD_COLS
    asset_pairs = client.reference_data_pair_metrics().first_page()[0]
    assert all([col in asset_pairs for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_institution_metrics() -> None:
    expected_cols = COMMON_REFD_COLS
    institution_data = client.reference_data_institution_metrics().first_page()[0]
    assert all([col in institution_data for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_exchange_asset_metrics() -> None:
    expected_cols = COMMON_REFD_COLS
    exchange_asset_metrics = client.reference_data_exchange_asset_metrics().first_page()[0]
    assert all([col in exchange_asset_metrics for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_exchange_metrics() -> None:
    expected_cols = COMMON_REFD_COLS
    exchange_asset_metrics = client.reference_data_exchange_metrics().first_page()[0]
    assert all([col in exchange_asset_metrics for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_asset_metrics() -> None:
    expected_cols = COMMON_REFD_COLS
    asset_metrics = client.reference_data_exchange_asset_metrics().first_page()[0]
    assert all([col in asset_metrics for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_assets() -> None:
    expected_cols = ["asset", "full_name"]
    assets = client.reference_data_assets().first_page()[0]
    assert all([col in assets for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_indexes() -> None:
    expected_cols = ["index", "full_name", "description", "type"]
    indexes = client.reference_data_indexes().first_page()[0]
    assert all([col in indexes for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_pairs() -> None:
    expected_cols = ["pair", "full_name"]
    pairs = client.reference_data_pairs().first_page()[0]
    assert all([col in pairs for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_market_metrics() -> None:
    expected_cols = ["metric", "full_name", "description", "product", "category", "subcategory", "unit", "data_type", "display_name"]
    pairs = client.reference_data_market_metrics().first_page()[0]
    assert all([col in pairs for col in expected_cols])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_refd_markets() -> None:
    expected_cols = ["market", "exchange", "type"]
    markets = client.reference_data_markets().first_page()[0]
    assert all([col in markets for col in expected_cols])


if __name__ == '__main__':
    pytest.main()




import pytest

from coinmetrics.api_client import CoinMetricsClient
from coinmetrics._utils import get_keys_from_catalog
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")), verbose=True)
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)

@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_trades() -> None:
    catalog_market_trades = client.catalog_market_trades_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_trades) == 10
    assert all(['market' in catalog for catalog in catalog_market_trades])
    df_market_trades = client.catalog_market_trades_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_trades = client.catalog_market_trades_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_trades[0])
    assert len(list_market_trades) > 0
    assert set(df_market_trades.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_trades() -> None:
    catalog_market_trades = client.catalog_full_market_trades_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_trades) == 10
    assert all(['market' in catalog for catalog in catalog_market_trades])
    df_market_trades = client.catalog_full_market_trades_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_trades = client.catalog_full_market_trades_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_trades[0])
    assert len(list_market_trades) > 0
    assert set(df_market_trades.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_candles() -> None:
    catalog_market_candles = client.catalog_full_market_candles_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_candles) == 10
    assert all(['market' in catalog for catalog in catalog_market_candles])
    assert all(['frequencies' in catalog for catalog in catalog_market_candles])
    df_market_candles = client.catalog_full_market_candles_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_candles = client.catalog_full_market_candles_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_candles[0])
    assert len(list_market_candles) > 0
    assert set(df_market_candles.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_candles() -> None:
    catalog_market_candles = client.catalog_market_candles_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_candles) == 10
    assert all(['market' in catalog for catalog in catalog_market_candles])
    assert all(['frequencies' in catalog for catalog in catalog_market_candles])
    df_market_candles = client.catalog_market_candles_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_candles = client.catalog_market_candles_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_candles[0])
    assert len(list_market_candles) > 0
    assert set(df_market_candles.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_contract_prices() -> None:
    catalog_contract_prices = client.catalog_market_contract_prices_v2(page_size=10, format='json').first_page()
    assert len(catalog_contract_prices) == 10
    assert all(['market' in catalog for catalog in catalog_contract_prices])
    assert all(['min_time' in catalog for catalog in catalog_contract_prices])
    assert all(['max_time' in catalog for catalog in catalog_contract_prices])
    df_market_contract_prices = client.catalog_market_contract_prices_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_dataframe()
    list_market_contract_prices = client.catalog_market_contract_prices_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_list()
    catalog_fields = get_keys_from_catalog(list_market_contract_prices[0])
    assert len(list_market_contract_prices) > 0
    assert set(df_market_contract_prices.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_contract_prices() -> None:
    catalog_contract_prices = client.catalog_full_market_contract_prices_v2(page_size=10, format='json').first_page()
    assert len(catalog_contract_prices) == 10
    assert all(['market' in catalog for catalog in catalog_contract_prices])
    assert all(['min_time' in catalog for catalog in catalog_contract_prices])
    assert all(['max_time' in catalog for catalog in catalog_contract_prices])
    df_market_contract_prices = client.catalog_full_market_contract_prices_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_dataframe()
    list_market_contract_prices = client.catalog_full_market_contract_prices_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_list()
    catalog_fields = get_keys_from_catalog(list_market_contract_prices[0])
    assert len(list_market_contract_prices) > 0
    assert set(df_market_contract_prices.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_funding_rates() -> None:
    catalog_funding_rates = client.catalog_full_market_funding_rates_v2(page_size=10, format='json').first_page()
    assert len(catalog_funding_rates) == 10
    assert all(['market' in catalog for catalog in catalog_funding_rates])
    assert all(['min_time' in catalog for catalog in catalog_funding_rates])
    assert all(['max_time' in catalog for catalog in catalog_funding_rates])
    df_market_funding_rates = client.catalog_full_market_funding_rates_v2(
        markets='binance-1000BONKUSDC-future'
    ).to_dataframe()
    list_market_funding_rates = client.catalog_full_market_funding_rates_v2(
        markets='binance-1000BONKUSDC-future'
    ).to_list()
    catalog_fields = get_keys_from_catalog(list_market_funding_rates[0])
    assert len(list_market_funding_rates) > 0
    assert set(df_market_funding_rates.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_funding_rates_predicted() -> None:
    catalog_funding_rates = client.catalog_full_market_funding_rates_predicted_v2(page_size=10, format='json').first_page()
    assert len(catalog_funding_rates) == 10
    assert all(['market' in catalog for catalog in catalog_funding_rates])
    assert all(['min_time' in catalog for catalog in catalog_funding_rates])
    assert all(['max_time' in catalog for catalog in catalog_funding_rates])
    df_market_funding_rates_predicted = client.catalog_full_market_funding_rates_predicted_v2(
        markets='bybit-10000000AIDOGEUSDT-future'
    ).to_dataframe()
    list_market_funding_rates_predicted = client.catalog_full_market_funding_rates_predicted_v2(
        markets='bybit-10000000AIDOGEUSDT-future'
    ).to_list()
    catalog_fields = get_keys_from_catalog(list_market_funding_rates_predicted[0])
    assert len(list_market_funding_rates_predicted) > 0
    assert set(df_market_funding_rates_predicted.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_funding_rates() -> None:
    catalog_funding_rates = client.catalog_market_funding_rates_v2(page_size=10, format='json').first_page()
    assert len(catalog_funding_rates) == 10
    assert all(['market' in catalog for catalog in catalog_funding_rates])
    assert all(['min_time' in catalog for catalog in catalog_funding_rates])
    assert all(['max_time' in catalog for catalog in catalog_funding_rates])
    df_market_funding_rates = client.catalog_market_funding_rates_v2(
        markets='binance-1000BONKUSDC-future'
    ).to_dataframe()
    list_market_funding_rates = client.catalog_market_funding_rates_v2(
        markets='binance-1000BONKUSDC-future'
    ).to_list()
    catalog_fields = get_keys_from_catalog(list_market_funding_rates[0])
    assert len(list_market_funding_rates) > 0
    assert set(df_market_funding_rates.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_funding_rates_predicted() -> None:
    catalog_funding_rates = client.catalog_market_funding_rates_predicted_v2(page_size=10, format='json').first_page()
    assert len(catalog_funding_rates) == 10
    assert all(['market' in catalog for catalog in catalog_funding_rates])
    assert all(['min_time' in catalog for catalog in catalog_funding_rates])
    assert all(['max_time' in catalog for catalog in catalog_funding_rates])
    df_market_funding_rates_predicted = client.catalog_market_funding_rates_predicted_v2(
        markets='bybit-10000000AIDOGEUSDT-future'
    ).to_dataframe()
    list_market_funding_rates_predicted = client.catalog_market_funding_rates_predicted_v2(
        markets='bybit-10000000AIDOGEUSDT-future'
    ).to_list()
    catalog_fields = get_keys_from_catalog(list_market_funding_rates_predicted[0])
    assert len(list_market_funding_rates_predicted) > 0
    assert set(df_market_funding_rates_predicted.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_greeks() -> None:
    catalog_greeks = client.catalog_market_greeks_v2(page_size=10, format='json').first_page()
    assert len(catalog_greeks) == 10
    assert all(['market' in catalog for catalog in catalog_greeks])
    assert all(['min_time' in catalog for catalog in catalog_greeks])
    assert all(['max_time' in catalog for catalog in catalog_greeks])
    df_market_greeks = client.catalog_market_greeks_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_dataframe()
    list_market_greeks = client.catalog_market_greeks_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_list()
    catalog_fields = get_keys_from_catalog(list_market_greeks[0])
    assert len(list_market_greeks) > 0
    assert set(df_market_greeks.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_greeks() -> None:
    catalog_greeks = client.catalog_full_market_greeks_v2(page_size=10, format='json').first_page()
    assert len(catalog_greeks) == 10
    assert all(['market' in catalog for catalog in catalog_greeks])
    assert all(['min_time' in catalog for catalog in catalog_greeks])
    assert all(['max_time' in catalog for catalog in catalog_greeks])
    df_market_greeks = client.catalog_full_market_greeks_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_dataframe()
    list_market_greeks = client.catalog_full_market_greeks_v2(markets="deribit-BTC-15OCT21-60000-C-option").to_list()
    catalog_fields = get_keys_from_catalog(list_market_greeks[0])
    assert len(list_market_greeks) > 0
    assert set(df_market_greeks.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_implied_volatility() -> None:
    catalog_market_implied_vol = client.catalog_full_market_implied_volatility_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_implied_vol) == 10
    assert all(['market' in catalog for catalog in catalog_market_implied_vol])
    assert all(['min_time' in catalog for catalog in catalog_market_implied_vol])
    assert all(['max_time' in catalog for catalog in catalog_market_implied_vol])
    df_market_implied_volatility = client.catalog_full_market_implied_volatility_v2(
        markets='binance-BNB-240620-550-C-option', page_size=10, format='json').to_dataframe()
    list_market_implied_volatility = client.catalog_full_market_implied_volatility_v2(
        markets='binance-BNB-240620-550-C-option', page_size=10, format='json').to_list()
    catalog_fields = get_keys_from_catalog(list_market_implied_volatility[0])
    assert len(list_market_implied_volatility) > 0
    assert set(df_market_implied_volatility.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_implied_volatility() -> None:
    catalog_market_implied_vol = client.catalog_market_implied_volatility_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_implied_vol) == 10
    assert all(['market' in catalog for catalog in catalog_market_implied_vol])
    assert all(['min_time' in catalog for catalog in catalog_market_implied_vol])
    assert all(['max_time' in catalog for catalog in catalog_market_implied_vol])
    df_market_implied_volatility = client.catalog_market_implied_volatility_v2(
        markets='binance-BNB-240620-550-C-option', page_size=10, format='json').to_dataframe()
    list_market_implied_volatility = client.catalog_market_implied_volatility_v2(
        markets='binance-BNB-240620-550-C-option', page_size=10, format='json').to_list()
    catalog_fields = get_keys_from_catalog(list_market_implied_volatility[0])
    assert len(list_market_implied_volatility) > 0
    assert set(df_market_implied_volatility.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_liquidations() -> None:
    catalog_market_liquidations = client.catalog_market_liquidations_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_liquidations) == 10
    assert all(['market' in catalog for catalog in catalog_market_liquidations])
    assert all(['min_time' in catalog for catalog in catalog_market_liquidations])
    assert all(['max_time' in catalog for catalog in catalog_market_liquidations])
    df_market_liquidations = client.catalog_market_liquidations_v2(markets="binance-BTCUSDT-future").to_dataframe()
    list_market_liquidations = client.catalog_market_liquidations_v2(markets="binance-BTCUSDT-future").to_list()
    catalog_fields = get_keys_from_catalog(list_market_liquidations[0])
    assert len(list_market_liquidations) > 0
    assert set(df_market_liquidations.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_liquidations() -> None:
    catalog_market_liquidations = client.catalog_full_market_liquidations_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_liquidations) == 10
    assert all(['market' in catalog for catalog in catalog_market_liquidations])
    assert all(['min_time' in catalog for catalog in catalog_market_liquidations])
    assert all(['max_time' in catalog for catalog in catalog_market_liquidations])
    df_market_liquidations = client.catalog_full_market_liquidations_v2(markets="binance-BTCUSDT-future").to_dataframe()
    list_market_liquidations = client.catalog_full_market_liquidations_v2(markets="binance-BTCUSDT-future").to_list()
    catalog_fields = get_keys_from_catalog(list_market_liquidations[0])
    assert len(list_market_liquidations) > 0
    assert set(df_market_liquidations.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_openinterest() -> None:
    catalog_market_openinterest = client.catalog_full_market_open_interest_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_openinterest) == 10
    assert all(['market' in catalog for catalog in catalog_market_openinterest])
    assert all(['min_time' in catalog for catalog in catalog_market_openinterest])
    assert all(['max_time' in catalog for catalog in catalog_market_openinterest])
    df_market_open_interest = client.catalog_full_market_open_interest_v2(markets="binance-BTCUSDT-future").to_dataframe()
    list_market_open_interest = client.catalog_full_market_open_interest_v2(markets="binance-BTCUSDT-future").to_list()
    catalog_fields = get_keys_from_catalog(list_market_open_interest[0])
    assert len(list_market_open_interest) > 0
    assert set(df_market_open_interest.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_openinterest() -> None:
    catalog_market_openinterest = client.catalog_market_open_interest_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_openinterest) == 10
    assert all(['market' in catalog for catalog in catalog_market_openinterest])
    assert all(['min_time' in catalog for catalog in catalog_market_openinterest])
    assert all(['max_time' in catalog for catalog in catalog_market_openinterest])
    df_market_open_interest = client.catalog_market_open_interest_v2(markets="binance-BTCUSDT-future").to_dataframe()
    list_market_open_interest = client.catalog_market_open_interest_v2(markets="binance-BTCUSDT-future").to_list()
    catalog_fields = get_keys_from_catalog(list_market_open_interest[0])
    assert len(list_market_open_interest) > 0
    assert set(df_market_open_interest.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_orderbooks() -> None:
    catalog_market_orderbooks = client.catalog_market_orderbooks_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_orderbooks) == 10
    assert all(['market' in catalog for catalog in catalog_market_orderbooks])
    assert all(['depths' in catalog for catalog in catalog_market_orderbooks])
    df_market_orderbooks = client.catalog_market_orderbooks_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_orderbooks = client.catalog_market_orderbooks_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_orderbooks[0])
    assert len(list_market_orderbooks) > 0
    assert set(df_market_orderbooks.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_quotes() -> None:
    catalog_market_quotes = client.catalog_market_quotes_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_quotes) == 10
    assert all(['market' in catalog for catalog in catalog_market_quotes])
    assert all(['min_time' in catalog for catalog in catalog_market_quotes])
    assert all(['max_time' in catalog for catalog in catalog_market_quotes])
    df_market_quotes = client.catalog_market_quotes_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_quotes = client.catalog_market_quotes_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_quotes[0])
    assert len(list_market_quotes) > 0
    assert set(df_market_quotes.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_quotes() -> None:
    catalog_market_quotes = client.catalog_full_market_quotes_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_quotes) == 10
    assert all(['market' in catalog for catalog in catalog_market_quotes])
    assert all(['min_time' in catalog for catalog in catalog_market_quotes])
    assert all(['max_time' in catalog for catalog in catalog_market_quotes])
    df_market_quotes = client.catalog_full_market_quotes_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    list_market_quotes = client.catalog_full_market_quotes_v2(markets="coinbase-btc-usd-spot").to_list()
    catalog_fields = get_keys_from_catalog(list_market_quotes[0])
    assert len(list_market_quotes) > 0
    assert set(df_market_quotes.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_orderbooks() -> None:
    catalog_market_orderbooks = client.catalog_full_market_orderbooks_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_orderbooks) == 10
    assert all(['market' in catalog for catalog in catalog_market_orderbooks])
    assert all(['depths' in catalog for catalog in catalog_market_orderbooks])
    list_market_orderbooks = client.catalog_full_market_orderbooks_v2(markets="coinbase-btc-usd-spot").to_list()
    df_market_orderbooks = client.catalog_full_market_orderbooks_v2(markets="coinbase-btc-usd-spot").to_dataframe()
    catalog_fields = get_keys_from_catalog(list_market_orderbooks[0])
    assert len(list_market_orderbooks) > 0
    assert set(df_market_orderbooks.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_market_metrics() -> None:
    catalog_market_metrics = client.catalog_full_market_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_metrics) == 10
    assert all(['market' in catalog for catalog in catalog_market_metrics])
    assert all(['metrics' in catalog for catalog in catalog_market_metrics])
    list_market_metrics = client.catalog_full_market_metrics_v2(
        markets='coinbase-btc-usd-spot', metrics='liquidity_slippage_10K_ask_percent'
    ).to_list()
    df_market_metrics = client.catalog_full_market_metrics_v2(
        markets='coinbase-btc-usd-spot', metrics='liquidity_slippage_10K_ask_percent'
    ).to_dataframe()
    catalog_fields = get_keys_from_catalog(list_market_metrics[0])
    assert len(list_market_metrics) > 0
    assert set(df_market_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_market_metrics() -> None:
    catalog_market_metrics = client.catalog_market_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_market_metrics) == 10
    assert all(['market' in catalog for catalog in catalog_market_metrics])
    assert all(['metrics' in catalog for catalog in catalog_market_metrics])
    list_market_metrics = client.catalog_market_metrics_v2(
        markets='coinbase-btc-usd-spot', metrics='liquidity_slippage_10K_ask_percent'
    ).to_list()
    df_market_metrics = client.catalog_market_metrics_v2(
        markets='coinbase-btc-usd-spot', metrics='liquidity_slippage_10K_ask_percent'
    ).to_dataframe()
    catalog_fields = get_keys_from_catalog(list_market_metrics[0])
    assert len(list_market_metrics) > 0
    assert set(df_market_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_asset_metrics() -> None:
    catalog_asset_metrics = client.catalog_asset_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_asset_metrics) == 10
    assert all(['asset' in catalog for catalog in catalog_asset_metrics])
    assert all(['metrics' in catalog for catalog in catalog_asset_metrics])
    list_asset_metrics = client.catalog_asset_metrics_v2(assets='btc', metrics='PriceUSD').to_list()
    df_asset_metrics = client.catalog_asset_metrics_v2(assets='btc', metrics='PriceUSD').to_dataframe()
    catalog_fields = get_keys_from_catalog(list_asset_metrics[0])
    assert len(list_asset_metrics) > 0
    assert set(df_asset_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_asset_metrics() -> None:
    catalog_asset_metrics = client.catalog_full_asset_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_asset_metrics) == 10
    assert all(['asset' in catalog for catalog in catalog_asset_metrics])
    assert all(['metrics' in catalog for catalog in catalog_asset_metrics])
    list_asset_metrics = client.catalog_full_asset_metrics_v2(assets='btc', metrics='PriceUSD').to_list()
    df_asset_metrics = client.catalog_full_asset_metrics_v2(assets='btc', metrics='PriceUSD').to_dataframe()
    catalog_fields = get_keys_from_catalog(list_asset_metrics[0])
    assert len(list_asset_metrics) > 0
    assert set(df_asset_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_metrics() -> None:
    catalog_exchange_metrics = client.catalog_exchange_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])
    list_exchange_metrics = client.catalog_exchange_metrics_v2(
        exchanges='binance', metrics='volume_reported_spot_usd_1h'
    ).to_list()
    df_exchange_metrics = client.catalog_exchange_metrics_v2(
        exchanges='binance',
        metrics='volume_reported_spot_usd_1h'
    ).to_dataframe()
    catalog_fields = get_keys_from_catalog(list_exchange_metrics[0])
    assert len(list_exchange_metrics) > 0
    assert set(df_exchange_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_exchange_metrics() -> None:
    catalog_exchange_metrics = client.catalog_full_exchange_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])
    list_exchange_metrics = client.catalog_full_exchange_metrics_v2(
        exchanges='binance', metrics='volume_reported_spot_usd_1h'
    ).to_list()
    df_exchange_metrics = client.catalog_full_exchange_metrics_v2(
        exchanges='binance',
        metrics='volume_reported_spot_usd_1h'
    ).to_dataframe()
    catalog_fields = get_keys_from_catalog(list_exchange_metrics[0])
    assert len(list_exchange_metrics) > 0
    assert set(df_exchange_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_asset_metrics() -> None:
    catalog_exchange_metrics = client.catalog_exchange_asset_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange_asset' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])
    list_exchange_asset_metrics = client.catalog_exchange_asset_metrics_v2(
        exchange_assets='binance-btc',
        metrics='volume_reported_spot_usd_1h'
    ).to_list()
    df_exchange_asset_metrics = client.catalog_exchange_asset_metrics_v2(
        exchange_assets='binance-btc',
        metrics='volume_reported_spot_usd_1h'
    ).to_dataframe()
    catalog_fields = get_keys_from_catalog(list_exchange_asset_metrics[0])
    assert len(list_exchange_asset_metrics) > 0
    assert set(df_exchange_asset_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_exchange_asset_metrics() -> None:
    catalog_exchange_metrics = client.catalog_full_exchange_asset_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_exchange_metrics) == 10
    assert all(['exchange_asset' in catalog for catalog in catalog_exchange_metrics])
    assert all(['metrics' in catalog for catalog in catalog_exchange_metrics])
    list_exchange_asset_metrics = client.catalog_full_exchange_asset_metrics_v2(
        exchange_assets='binance-btc',
        metrics='volume_reported_spot_usd_1h'
    ).to_list()
    df_exchange_asset_metrics = client.catalog_full_exchange_asset_metrics_v2(
        exchange_assets='binance-btc',
        metrics='volume_reported_spot_usd_1h'
    ).to_dataframe()
    catalog_fields = get_keys_from_catalog(list_exchange_asset_metrics[0])
    assert len(list_exchange_asset_metrics) > 0
    assert set(df_exchange_asset_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_pair_metrics() -> None:
    catalog_pair_metrics = client.catalog_pair_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_pair_metrics) == 10
    assert all(['pair' in catalog for catalog in catalog_pair_metrics])
    assert all(['metrics' in catalog for catalog in catalog_pair_metrics])
    list_pair_metrics = client.catalog_pair_metrics_v2(pairs='btc-usd', metrics='volume_reported_spot_usd_1h').to_list()
    df_pair_metrics = client.catalog_pair_metrics_v2(pairs='btc-usd', metrics='volume_reported_spot_usd_1h').to_dataframe()
    catalog_fields = get_keys_from_catalog(list_pair_metrics[0])
    assert len(list_pair_metrics) > 0
    assert set(df_pair_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_exchange_full_pair_metrics() -> None:
    catalog_pair_metrics = client.catalog_full_pair_metrics_v2(page_size=10, format='json').first_page()
    assert len(catalog_pair_metrics) == 10
    assert all(['pair' in catalog for catalog in catalog_pair_metrics])
    assert all(['metrics' in catalog for catalog in catalog_pair_metrics])
    list_pair_metrics = client.catalog_full_pair_metrics_v2(pairs='btc-usd', metrics='volume_reported_spot_usd_1h').to_list()
    df_pair_metrics = client.catalog_full_pair_metrics_v2(pairs='btc-usd', metrics='volume_reported_spot_usd_1h').to_dataframe()
    catalog_fields = get_keys_from_catalog(list_pair_metrics[0])
    assert len(list_pair_metrics) > 0
    assert set(df_pair_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_institution_pair_metrics() -> None:
    catalog_institution_metrics = client.catalog_institution_metrics_v2(page_size=10, format='json').first_page()
    assert all(['institution' in catalog for catalog in catalog_institution_metrics])
    assert all(['metrics' in catalog for catalog in catalog_institution_metrics])
    list_institution_metrics = client.catalog_institution_metrics_v2(institutions='grayscale').to_list()
    df_institution_metrics = client.catalog_institution_metrics_v2(institutions='grayscale').to_dataframe()
    catalog_fields = get_keys_from_catalog(list_institution_metrics[0])
    assert len(list_institution_metrics) > 0
    assert set(df_institution_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_full_institution_pair_metrics() -> None:
    catalog_institution_metrics = client.catalog_full_institution_metrics_v2(page_size=10, format='json').first_page()
    assert all(['institution' in catalog for catalog in catalog_institution_metrics])
    assert all(['metrics' in catalog for catalog in catalog_institution_metrics])
    list_institution_metrics = client.catalog_full_institution_metrics_v2(institutions='grayscale').to_list()
    df_institution_metrics = client.catalog_full_institution_metrics_v2(institutions='grayscale').to_dataframe()
    catalog_fields = get_keys_from_catalog(list_institution_metrics[0])
    assert len(list_institution_metrics) > 0
    assert set(df_institution_metrics.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_pair_candles() -> None:
    catalog_pair_candles = client.catalog_pair_candles_v2(page_size=10, format='json').first_page()
    catalog_full_pair_candles = client.catalog_full_pair_candles_v2(page_size=10, format='json').first_page()
    assert all(['pair' in catalog for catalog in catalog_pair_candles])
    assert all(['pair' in catalog for catalog in catalog_full_pair_candles])
    assert all(['frequencies' in catalog for catalog in catalog_pair_candles])
    assert all(['frequencies' in catalog for catalog in catalog_full_pair_candles])
    df_candles = client.catalog_pair_candles_v2(pairs="btc-usd").to_dataframe()
    list_candles = client.catalog_pair_candles_v2(pairs="btc-usd").to_list()
    catalog_fields = get_keys_from_catalog(list_candles[0])
    assert len(list_candles) > 0
    assert set(df_candles.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_index_candles() -> None:
    catalog_index_candles = client.catalog_index_candles_v2(page_size=10, format='json').first_page()
    catalog_full_index_candles = client.catalog_full_index_candles_v2(page_size=10, format='json').first_page()
    assert all(['index' in catalog for catalog in catalog_index_candles])
    assert all(['index' in catalog for catalog in catalog_full_index_candles])
    assert all(['frequencies' in catalog for catalog in catalog_index_candles])
    assert all(['frequencies' in catalog for catalog in catalog_full_index_candles])
    df_candles = client.catalog_index_candles_v2(indexes="CMBI10").to_dataframe()
    list_candles = client.catalog_index_candles_v2(indexes="CMBI10").to_list()
    catalog_fields = get_keys_from_catalog(list_candles[0])
    assert len(list_candles) > 0
    assert set(df_candles.columns) == catalog_fields


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_asset_chains() -> None:
    catalog_asset_chains = client.catalog_asset_chains_v2(page_size=10, format='json').first_page()
    catalog_full_asset_chains = client.catalog_full_asset_chains_v2(page_size=10, format='json').first_page()
    assert all(['asset' in catalog for catalog in catalog_asset_chains])
    assert all(['asset' in catalog for catalog in catalog_full_asset_chains])
    assert all(['min_time' in catalog for catalog in catalog_asset_chains])
    assert all(['min_time' in catalog for catalog in catalog_full_asset_chains])
    assert all(['max_time' in catalog for catalog in catalog_asset_chains])
    assert all(['max_time' in catalog for catalog in catalog_full_asset_chains])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_mempool_feerates() -> None:
    catalog_mempool_feerates = client.catalog_mempool_feerates_v2(page_size=10, format='json').first_page()
    catalog_full_mempool_feerates = client.catalog_full_mempool_feerates_v2(page_size=10, format='json').first_page()
    assert all(['asset' in catalog for catalog in catalog_mempool_feerates])
    assert all(['asset' in catalog for catalog in catalog_full_mempool_feerates])
    assert all(['min_time' in catalog for catalog in catalog_mempool_feerates])
    assert all(['min_time' in catalog for catalog in catalog_full_mempool_feerates])
    assert all(['max_time' in catalog for catalog in catalog_mempool_feerates])
    assert all(['max_time' in catalog for catalog in catalog_full_mempool_feerates])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_mining_pool_tips() -> None:
    catalog_mining_pool_tips = client.catalog_asset_chains_v2(page_size=10, format='json').first_page()
    catalog_full_mining_pool_tips = client.catalog_full_asset_chains_v2(page_size=10, format='json').first_page()
    assert all(['asset' in catalog for catalog in catalog_mining_pool_tips])
    assert all(['asset' in catalog for catalog in catalog_full_mining_pool_tips])
    assert all(['min_time' in catalog for catalog in catalog_mining_pool_tips])
    assert all(['min_time' in catalog for catalog in catalog_full_mining_pool_tips])
    assert all(['max_time' in catalog for catalog in catalog_mining_pool_tips])
    assert all(['max_time' in catalog for catalog in catalog_full_mining_pool_tips])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_transaction_tracker() -> None:
    catalog_transaction_tracker = client.catalog_transaction_tracker_assets_v2(page_size=10, format='json').first_page()
    catalog_full_transaction_tracker = client.catalog_full_transaction_tracker_assets_v2(page_size=10, format='json').first_page()
    assert all(['asset' in catalog for catalog in catalog_transaction_tracker])
    assert all(['asset' in catalog for catalog in catalog_full_transaction_tracker])
    assert all(['min_time' in catalog for catalog in catalog_transaction_tracker])
    assert all(['min_time' in catalog for catalog in catalog_full_transaction_tracker])
    assert all(['max_time' in catalog for catalog in catalog_transaction_tracker])
    assert all(['max_time' in catalog for catalog in catalog_full_transaction_tracker])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_index_levels() -> None:
    catalog_index_levels = client.catalog_index_levels_v2(page_size=10, format='json').first_page()
    catalog_all_index_levels = client.catalog_index_levels_v2(page_size=10, format='json').first_page()
    assert all(['index' in catalog for catalog in catalog_index_levels])
    assert all(['index' in catalog for catalog in catalog_all_index_levels])
    assert all(['frequencies' in catalog for catalog in catalog_index_levels])
    assert all(['frequencies' in catalog for catalog in catalog_all_index_levels])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_blockchain_blocks() -> None:
    catalog_blockchain_blocks = client.catalog_blockchain_blocks_v2()
    catalog_full_blockchain_blocks = client.catalog_full_blockchain_blocks_v2()
    assert all(['asset' in catalog for catalog in catalog_blockchain_blocks])
    assert all(['asset' in catalog for catalog in catalog_full_blockchain_blocks])
    assert all(['min_time' in catalog for catalog in catalog_blockchain_blocks])
    assert all(['min_time' in catalog for catalog in catalog_full_blockchain_blocks])
    assert all(['max_time' in catalog for catalog in catalog_blockchain_blocks])
    assert all(['max_time' in catalog for catalog in catalog_full_blockchain_blocks])
    
    
@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_blockchain_accounts() -> None:
    catalog_blockchain_accounts = client.catalog_blockchain_accounts_v2()
    catalog_full_blockchain_accounts = client.catalog_full_blockchain_accounts_v2()
    assert all(['asset' in catalog for catalog in catalog_blockchain_accounts])
    assert all(['asset' in catalog for catalog in catalog_full_blockchain_accounts])
    assert all(['min_time' in catalog for catalog in catalog_blockchain_accounts])
    assert all(['min_time' in catalog for catalog in catalog_full_blockchain_accounts])
    assert all(['max_time' in catalog for catalog in catalog_blockchain_accounts])
    assert all(['max_time' in catalog for catalog in catalog_full_blockchain_accounts])
    
    
@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_blockchain_balance_updates() -> None:
    catalog_blockchain_balance_updates = client.catalog_blockchain_balance_updates_v2()
    catalog_full_blockchain_balance_updates = client.catalog_full_blockchain_balance_updates_v2()
    assert all(['asset' in catalog for catalog in catalog_blockchain_balance_updates])
    assert all(['asset' in catalog for catalog in catalog_full_blockchain_balance_updates])
    assert all(['min_time' in catalog for catalog in catalog_blockchain_balance_updates])
    assert all(['min_time' in catalog for catalog in catalog_full_blockchain_balance_updates])
    assert all(['max_time' in catalog for catalog in catalog_blockchain_balance_updates])
    assert all(['max_time' in catalog for catalog in catalog_full_blockchain_balance_updates])
    
    
@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalogv2_blockchain_transactions() -> None:
    catalog_blockchain_transactions = client.catalog_blockchain_transactions_v2()
    catalog_full_blockchain_transactions = client.catalog_full_blockchain_transactions_v2()
    assert all(['asset' in catalog for catalog in catalog_blockchain_transactions])
    assert all(['asset' in catalog for catalog in catalog_full_blockchain_transactions])
    assert all(['min_time' in catalog for catalog in catalog_blockchain_transactions])
    assert all(['min_time' in catalog for catalog in catalog_full_blockchain_transactions])
    assert all(['max_time' in catalog for catalog in catalog_blockchain_transactions])
    assert all(['max_time' in catalog for catalog in catalog_full_blockchain_transactions])


if __name__ == '__main__':
    pytest.main()


import datetime

import pytest

from coinmetrics.api_client import CoinMetricsClient
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")), verbose=True)
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_blockchain_list_of_account_balance_updates() -> None:
    asset = "btc"
    account = "112jmDkNGHSbhhY17JGpxU3sMA9ZExG7b2"
    updates = client.get_list_of_balance_updates_for_account_v2(asset=asset, account=account).to_list()
    assert updates[0]['block_hash'] == "0000000000000000002c7505ef2272e0677fa53d68d633f8e076ed42dd3380e6"


if __name__ == "__main__":
    pytest.main()



import pytest

from coinmetrics.api_client import CoinMetricsClient
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")))
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_asset_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_asset_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_exchange_asset_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_exchange_asset_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_pair_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_pair_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_exchange_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_exchange_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_institution_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_institution_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_market_metrics() -> None:
    """
    Tests the catalog/market-metrics endpoint works as expected - some data is returned and when converted to dataframe
    it has the expected amount of rows (5)
    """
    data = client.catalog_market_metrics()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns == 5)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_asset_pair_candles() -> None:
    """
    Tests the catalog-full/pair-candles endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (4)
    """
    data = client.catalog_full_asset_pair_candles()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df != 0)
    assert len(data_df.columns) == 4


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_market_trades() -> None:
    """
    Tests the catalog-full/market-trades endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_full_market_trades()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 3
    expected_cols = ["market", "max_time", "min_time"]
    assert all(col in expected_cols for col in data_df.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_market_orderbooks() -> None:
    """
    Tests the catalog-full/market-orderbooks endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_full_market_orderbooks()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 4
    expected_cols = ["market", "max_time", "min_time", "depths"]
    assert all(col in expected_cols for col in data_df.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_market_quotes() -> None:
    """
    Tests the catalog-full/market-quotes endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_full_market_quotes()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 3
    expected_cols = ["market", "max_time", "min_time"]
    assert all(col in expected_cols for col in data_df.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_market_greeks() -> None:
    """
    Tests the catalog-full/market-quotes endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_market_greeks()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 3
    expected_cols = ["market", "max_time", "min_time"]
    assert all(col in expected_cols for col in data_df.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_market_funding_rates() -> None:
    """
    Tests the catalog-full/market-funding-rates endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_full_market_funding_rates()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 3
    expected_cols = ["market", "max_time", "min_time"]
    assert all(col in expected_cols for col in data_df.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_market_openinterest() -> None:
    """
    Tests the catalog-full/market-openinterest endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_full_market_open_interest()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 3
    expected_cols = ["market", "max_time", "min_time"]
    assert all(col in expected_cols for col in data_df.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_catalog_full_market_liquidations() -> None:
    """
    Tests the catalog-full/market-openinterest endpoints works as expected - data is returned and when converted to dataframe
    it has the expected number of columns (3)
    """
    data = client.catalog_full_market_liquidations()
    assert len(data) != 0
    data_df = data.to_dataframe()
    assert len(data_df) != 0
    assert len(data_df.columns) == 3
    expected_cols = ["market", "max_time", "min_time"]
    assert all(col in expected_cols for col in data_df.columns)


# type: ignore
import datetime
from datetime import timedelta
import dateutil.relativedelta
import pandas as pd
import pytest

from coinmetrics.api_client import CoinMetricsClient
from coinmetrics._data_collection import ParallelDataCollection
import os

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")))
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_market_trades() -> None:
    """
    Basic test to make sure that parallel market trades works over a short horizon
    """
    markets = ['coinbase-eth-usdc-spot', 'coinbase-eth-btc-spot']
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=1, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    parallel_trades = client.get_market_trades(markets=markets, start_time=start, end_time=end).parallel().to_list()
    normal_trades = client.get_market_trades(markets=markets, start_time=start, end_time=end).to_list()
    unique_markets = set([trade['market'] for trade in parallel_trades])
    assert len(parallel_trades) > 1
    assert len(parallel_trades) == len(normal_trades)
    for market in markets:
        assert market in unique_markets


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_market_trades_to_df() -> None:
    """
    Basic test to make sure to_dataframe renders right
    """
    markets = ['coinbase-eth-usdc-spot', 'coinbase-eth-btc-spot']
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=1, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    parallel_trades = client.get_market_trades(markets=markets, start_time=start,
                                               end_time=end).parallel().to_dataframe()
    normal_trades = client.get_market_trades(markets=markets, start_time=start, end_time=end).to_dataframe()
    assert len(parallel_trades) > 1
    assert len(normal_trades) == len(parallel_trades)
    assert all([market in set(normal_trades['market']) for market in markets])


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_asset_metrics_to_single_csv() -> None:
    metrics = ["volume_reported_spot_usd_1d", "volume_trusted_spot_usd_1d"]
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=4, month=6, hour=3, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    normal_data = client.get_asset_metrics(assets="btc", metrics=metrics, start_time=start, end_time=end).to_list()
    _export = client.get_asset_metrics(assets="btc", metrics=metrics, start_time=start, end_time=end).parallel(
        parallelize_on="metrics").export_to_csv("metrics.csv")
    parallel_data = pd.read_csv("metrics.csv")
    assert len(normal_data) == len(parallel_data)
    assert len(parallel_data) == 4


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_export_to_csvs() -> None:
    """
    Tests that creates the expected files
    """
    assets = ["btc", "eth", "algo"]
    expected_files = [f"asset-metrics/{asset}.csv" for asset in assets]
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    try:
        _export_asset_metrics_csvs = client.get_asset_metrics(
            assets=assets,
            metrics="ReferenceRateUSD",
            frequency="1m",
            start_time=start,
            end_time=end
        ).parallel(parallelize_on="assets").export_to_csv_files()
        assert all([os.path.exists(file) for file in expected_files])
        assert ([len(pd.read_csv(file)) > 5 for file in expected_files])

    finally:
        for file in expected_files:
            if os.path.exists(file):
                os.remove(file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_export_to_single_csv() -> None:
    """
    Tests that creates the expected
    """
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    expected_file = "asset_metrics.csv"
    try:
        export_to_csv = client.get_asset_metrics(assets=["btc", "eth", "algo"],
                                                 metrics="ReferenceRateUSD",
                                                 frequency="1m",
                                                 start_time=start,
                                                 end_time=end).parallel().export_to_csv("asset_metrics.csv")
        normal_data = client.get_asset_metrics(assets=["btc", "eth", "algo"],
                                               metrics="ReferenceRateUSD",
                                               frequency="1m",
                                               start_time=start,
                                               end_time=end).to_list()
        df_from_csv = pd.read_csv(expected_file)
        assert os.path.exists(expected_file)
        assert len(df_from_csv) > 5
        assert len(normal_data) == len(df_from_csv)
    finally:
        os.remove(expected_file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_export_to_json_files() -> None:
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    assets = ["btc", "eth", "algo"]
    expected_files = [f"asset-metrics/{asset}.json" for asset in assets]
    try:
        _export_asset_metrics_json = client.get_asset_metrics(
            assets=assets,
            metrics="ReferenceRateUSD",
            frequency="1m",
            start_time=start,
            end_time=end
        ).parallel().export_to_json_files()
        assert all([os.path.exists(file) for file in expected_files])
        assert ([len(pd.read_json(file, lines=True)) > 5 for file in expected_files])

    finally:
        for file in expected_files:
            if os.path.exists(file):
                os.remove(file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_export_to_single_json() -> None:
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    expected_file = "asset-metrics.json"
    try:
        _export_to_json = client.get_asset_metrics(
            assets=["btc", "eth", "algo"],
            metrics="ReferenceRateUSD",
            frequency="1m",
            start_time=start,
            end_time=end
        ).parallel().export_to_json(expected_file)
        normal_data = client.get_asset_metrics(
            assets=["btc", "eth", "algo"],
            metrics="ReferenceRateUSD",
            frequency="1m",
            start_time=start,
            end_time=end
        ).to_list()
        _df_from_csv = pd.read_json(expected_file, lines=True)
        assert os.path.exists(expected_file)
        assert len(pd.read_json(expected_file, lines=True)) > 5
        assert len(normal_data) == len(pd.read_json(expected_file, lines=True))
    finally:
        os.remove(expected_file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallelize_on_metrics() -> None:
    metrics = ["volume_reported_spot_usd_1d", "volume_trusted_spot_usd_1d"]
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=4, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    normal_data = client.get_asset_metrics(assets="btc", metrics=metrics, start_time=start, end_time=end).to_dataframe()
    parallel_data = client.get_asset_metrics(assets="btc", metrics=metrics, start_time=start, end_time=end).parallel(
        parallelize_on="metrics").to_dataframe()
    assert len(normal_data) == len(parallel_data)
    assert len(parallel_data) == 4


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallelize_on_metrics_export_to_csvs() -> None:
    metrics = ["volume_reported_spot_usd_1d", "volume_trusted_spot_usd_1d"]
    expected_files = [f"asset-metrics/{m}.csv" for m in metrics]
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=4, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    try:
        client.get_asset_metrics(
            assets="btc",
            metrics=metrics,
            start_time=start,
            end_time=end
        ).parallel(
            parallelize_on="metrics"
        ).export_to_csv_files()
        for file in expected_files:
            assert os.path.exists(file)
            assert len(pd.read_csv(file)) == 4
    finally:
        for file in expected_files:
            if os.path.exists(file):
                os.remove(file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallelize_time_increment_export_to_json_files() -> None:
    metrics = "ReferenceRateUSD"
    assets = ["btc", "eth"]
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=2, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    expected_times = [
        datetime.datetime(year=2022, day=1, month=6, hour=h, minute=0, second=0).strftime("%Y-%m-%dT%H-%M-%SZ")
        for h in range(0, 24, 4)
    ]
    expected_files = [
        f"asset-metrics/{asset}/start_time={time}.json"
        for time in expected_times
        for asset in assets
    ]
    try:
        client.get_asset_metrics(
            assets=assets,
            metrics=metrics,
            start_time=start,
            end_time=end,
            frequency="1h"
        ).parallel(
            parallelize_on="assets",
            time_increment=timedelta(hours=4)
        ).export_to_json_files()
        for file in expected_files:
            assert os.path.exists(file)
            assert len(pd.read_json(file, lines=True)) == 4
    finally:
        for file in expected_files:
            if os.path.exists(file):
                os.remove(file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallelize_on_metrics_and_assets_export_to_csvs() -> None:
    metrics = ["volume_reported_spot_usd_1d", "volume_trusted_spot_usd_1d"]
    assets = ["btc", "eth"]
    start = datetime.datetime(year=2022, day=1, month=6, hour=0, minute=0, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    end = datetime.datetime(year=2022, day=4, month=6, hour=0, minute=20, second=0).strftime("%Y-%m-%dT%H:%M:%SZ")
    expected_files = [f"asset-metrics/{asset}/{metric}.csv" for asset in assets for metric in metrics]
    try:
        client.get_asset_metrics(assets=assets, metrics=metrics, start_time=start, end_time=end).parallel(
            parallelize_on=["assets", "metrics"]).export_to_csv_files()
        for file in expected_files:
            assert os.path.exists(file)
            assert len(pd.read_csv(file)) == 4
    finally:
        for file in expected_files:
            if os.path.exists(file):
                os.remove(file)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_to_dataframe_time_increment_month() -> None:
    markets = ['coinbase-eth-usdc-spot']
    test_df_parallel = client.get_market_candles(
        markets=markets,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False
    ).parallel(time_increment=dateutil.relativedelta.relativedelta(months=1)).to_dataframe()
    test_df_normal = client.get_market_candles(
        markets=markets,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False
    ).to_dataframe()
    assert len(test_df_parallel) > 10
    assert len(test_df_normal) == len(test_df_parallel)
    assert set(test_df_parallel.columns) == set(test_df_normal.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_to_dataframe_time_increment_week() -> None:
    markets = ['coinbase-eth-usdc-spot']
    test_df_parallel = client.get_market_candles(
        markets=markets,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False
    ).parallel(
        progress_bar=False,
        time_increment=dateutil.relativedelta.relativedelta(weeks=1)
    ).to_dataframe()
    test_df_normal = client.get_market_candles(
        markets=markets,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False
    ).to_dataframe()
    assert len(test_df_parallel) > 10
    assert len(test_df_normal) == len(test_df_parallel)
    assert set(test_df_parallel.columns) == set(test_df_normal.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_to_dataframe_time_increment_week_multiple_markets() -> None:
    markets = ['coinbase-eth-usdc-spot', 'coinbase-eth-btc-spot']
    test_df_parallel = client.get_market_candles(
        markets=markets,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False
    ).parallel(
        progress_bar=False,
        time_increment=dateutil.relativedelta.relativedelta(weeks=1)
    ).to_dataframe()
    test_df_normal = client.get_market_candles(
        markets=markets,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False
    ).to_dataframe()
    assert len(test_df_parallel) > 10
    assert len(test_df_normal) == len(test_df_parallel)
    assert set(test_df_parallel.columns) == set(test_df_normal.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parallel_asset_metrics_using_time_increment_enum() -> None:
    metrics = ["ReferenceRateUSD", "AdrActCnt"]
    test_df_parallel = client.get_asset_metrics(
        assets="btc",
        metrics=metrics,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False,
        frequency="1d"
    ).parallel(
        parallelize_on=["metrics"],
        time_increment=dateutil.relativedelta.relativedelta(months=1)
    ).to_dataframe()
    test_df_normal = client.get_asset_metrics(
        assets="btc",
        metrics=metrics,
        page_size=1000,
        start_time="2022-01-01",
        end_time="2022-03-01",
        end_inclusive=False,
        frequency="1d"
    ).to_dataframe()
    assert len(test_df_parallel) > 10
    assert len(test_df_normal) == len(test_df_parallel)
    assert set(test_df_parallel.columns) == set(test_df_normal.columns)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_export_to_csvs_time_increment_files() -> None:
    assets = ["btc"]
    client.get_asset_metrics(
        assets=assets,
        metrics="ReferenceRateUSD",
        frequency="1m",
        start_time="2023-03-01",
        end_time="2023-05-01",
        page_size=1000,
        end_inclusive=False
    ).parallel(
        time_increment=timedelta(days=1)
    ).export_to_csv("btcRRs.csv")


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_blockchain_parallel_export() -> None:
    df_temp = client.get_list_of_blocks_v2(
        asset="btc",
        start_time='2023-09-07',
        end_time='2023-09-22',
        page_size=10000,
        end_inclusive=False
    ).parallel(time_increment=timedelta(days=1)).to_dataframe()
    assert len(df_temp) > 2000


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_parse_date() -> None:
    pdc = ParallelDataCollection
    t_naive_date = "2024-01-02"
    t_naive_digit = "20240102"
    t_naive_s = "2024-01-02T00:03:04"
    t_naive_sz = "2024-01-02T00:03:04Z"
    assert pdc.parse_date(t_naive_date) == datetime.datetime(year=2024, month=1, day=2)
    assert pdc.parse_date(t_naive_digit) == datetime.datetime(year=2024, month=1, day=2)
    assert pdc.parse_date(t_naive_s) == datetime.datetime(year=2024, month=1, day=2, hour=0, minute=3, second=4)
    assert pdc.parse_date(t_naive_sz) == datetime.datetime(year=2024, month=1, day=2, hour=0, minute=3, second=4)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_height_increment() -> None:
    df_blocks = client.get_list_of_blocks_v2(
        asset="btc",
        start_height=10,
        end_height=100,
        end_inclusive=False
    ).parallel(height_increment=10).to_dataframe()
    assert len(df_blocks) == 90


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_end_time_undefined() -> None:
    start_time = datetime.datetime.utcnow().replace(microsecond=0, second=0) - timedelta(minutes=60)
    df_metrics_1m = client.get_asset_metrics(
        assets='btc',
        metrics='ReferenceRateUSD',
        start_time=start_time,
        frequency='1m'
    ).parallel(time_increment=timedelta(minutes=1)).to_dataframe()
    assert not df_metrics_1m.empty
    assert df_metrics_1m.time.min().to_pydatetime().replace(tzinfo=None) == start_time


if __name__ == '__main__':
    pytest.main()



import pandas as pd
import pytest
from coinmetrics.api_client import CoinMetricsClient
import os
import numpy as np

CM_API_KEY = os.environ.get("CM_API_KEY")
client = CoinMetricsClient(str(CM_API_KEY))
cm_api_key_set = CM_API_KEY is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_defi_balance_sheets() -> None:
    """
    Testing this works for aave
    """
    aave_balance_sheets = client.get_defi_balance_sheets(
        defi_protocols="aave_v2_eth"
    ).to_dataframe()
    assert len(aave_balance_sheets) > 100
    assert aave_balance_sheets.iloc[:1]["defi_protocol"][0] == "aave_v2_eth"


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_transaction_tracker_catalog_full() -> None:
    data = client.catalog_full_transaction_tracker_assets().to_dataframe()
    assert len(data) >= 2


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_transaction_tracker_catalog() -> None:
    data = client.catalog_transaction_tracker_assets().to_dataframe()
    assert len(data) >= 2


if __name__ == "__main__":
    pytest.main()



# type: ignore
import pytest
import orjson
import os
import websocket
from coinmetrics.api_client import CoinMetricsClient, CmStream
import signal

client = CoinMetricsClient(str(os.environ.get("CM_API_KEY")))
cm_api_key_set = os.environ.get("CM_API_KEY") is not None
REASON_TO_SKIP = "Need to set CM_API_KEY as an env var in order to run this test"

print("CM_API_KEY is set - tests will run") if cm_api_key_set else print(
    "CM_API_KEY not set, tests will not run"
)


def on_message_index_levels_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = ["index", "time", "level", "cm_sequence_id"]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_market_trades_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = [
        "market",
        "time",
        "coin_metrics_id",
        "amount",
        "price",
        "collect_time",
        "side",
        "cm_sequence_id",
    ]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_market_orderbooks_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = [
        "market",
        "time",
        "coin_metrics_id",
        "asks",
        "bids",
        "type",
        "collect_time",
        "cm_sequence_id",
    ]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_market_openinterest(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = [
        "market",
        "time",
        "contract_count",
        "value_usd",
        "cm_sequence_id"
    ]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_market_candles_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = [
        "market",
        "time",
        "price_open",
        "price_close",
        "price_high",
        "price_low",
        "vwap",
        "volume",
        "candle_usd_volume",
        "candle_trades_count",
        "cm_sequence_id",
    ]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_market_quotes_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = [
        "market",
        "time",
        "coin_metrics_id",
        "ask_price",
        "ask_size",
        "bid_price",
        "bid_size",
        "cm_sequence_id",
    ]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)
    

def on_message_asset_metrics_rr_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests that data with the expected keys can be loaded by orjson
    """
    data = orjson.loads(message)
    expected_cols_index_levels = ["time", "asset", "ReferenceRateUSD", "cm_sequence_id"]
    for col in expected_cols_index_levels:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_pair_quotes_test(stream: websocket.WebSocketApp, message: str) -> None:
    data = orjson.loads(message)
    expected_cols = ["pair", "time", "ask_price", "mid_price"]
    for col in expected_cols:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_assets_quotes_test(stream: websocket.WebSocketApp, message: str) -> None:
    data = orjson.loads(message)
    expected_cols = ["pair", "time", "ask_price", "ask_size", "bid_price", "mid_price"]
    for col in expected_cols:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_market_liquidations_test(stream: websocket.WebSocketApp, message: str) -> None:
    data = orjson.loads(message)
    expected_cols = [
        "market",
        "time",
        "coin_metrics_id",
        "price",
        "type",
        "side",
        "cm_sequence_id"
    ]
    for col in expected_cols:
        assert col in data
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 0:
        os.kill(os.getpid(), signal.SIGINT)


def on_message_sigterm_test(stream: websocket.WebSocketApp, message: str) -> None:
    """
    Tests closing websocket using sigterm
    """
    data = orjson.loads(message)
    sequence_id = int(data['cm_sequence_id'])
    if sequence_id >= 1:
        stream.close()


def on_close_sigterm_test(stream: websocket.WebSocketApp, close_status_code: int, close_msg: str) -> None:
    os.kill(os.getpid(), signal.SIGTERM)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_index_levels_stream() -> None:
    stream = client.get_stream_index_levels(indexes="*")
    stream.run(on_message=on_message_index_levels_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_market_trades_stream() -> None:
    markets = ["*"]
    stream = client.get_stream_market_trades(markets=markets)
    stream.run(on_message=on_message_market_trades_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_market_orderbooks_stream() -> None:
    markets = ["*"]
    stream = client.get_stream_market_orderbooks(markets=markets)
    stream.run(on_message=on_message_market_orderbooks_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_market_quotes_stream() -> None:
    markets = ["*"]
    stream = client.get_stream_market_quotes(markets=markets)
    stream.run(on_message_market_quotes_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_asset_metrics() -> None:
    metrics = ["ReferenceRateUSD"]
    stream = client.get_stream_asset_metrics(
        assets="*", metrics=metrics, frequency="1s"
    )
    stream.run(on_message_asset_metrics_rr_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_market_candles_stream() -> None:
    stream = client.get_stream_market_candles(
        markets="*", frequency="1m"
    )
    stream.run(on_message_market_candles_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_pair_quotes_stream() -> None:
    stream = client.get_stream_pair_quotes(pairs="btc-*")
    stream.run(on_message=on_message_pair_quotes_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_asset_quotes_stream() -> None:
    stream = client.get_stream_asset_quotes(assets="*")
    stream.run(on_message=on_message_assets_quotes_test)

@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_market_liquidations() -> None:
    stream = client.get_stream_market_liquidations(markets='*')
    stream.run(on_message=on_message_market_liquidations_test)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_get_market_openinterest() -> None:
    stream = client.get_stream_market_open_interest(markets='*')
    stream.run(on_message=on_message_market_openinterest)


@pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
def test_on_close() -> None:
    on_close_ran = False
    def on_close(stream: websocket.WebSocketApp, close_status_code: int, close_msg: str) -> None:
        nonlocal on_close_ran
        on_close_ran = True
    stream = client.get_stream_asset_quotes(assets="btc")
    stream.run(on_message=on_message_assets_quotes_test, on_close=on_close)
    assert on_close_ran
    
    
# @pytest.mark.skipif(not cm_api_key_set, reason=REASON_TO_SKIP)
# def test_sigterm() -> None:
#     assets = ["btc"]
#     metrics = ["ReferenceRateUSD"]
#     stream = client.get_stream_asset_metrics(
#         assets=assets, metrics=metrics, frequency="1s"
#     )
#     stream.run(on_message=on_message_sigterm_test, on_close=on_close_sigterm_test)


if __name__ == '__main__':
    pytest.main()